"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const checkIfSection_1 = __importDefault(require("./checkIfSection"));
const convertTo1DArray_1 = __importDefault(require("./convertTo1DArray"));
const convertTo2DArray_1 = __importDefault(require("./convertTo2DArray"));
const mapMatrix_1 = __importDefault(require("./mapMatrix"));
const transposeMatrix_1 = __importDefault(require("./transposeMatrix"));
const checkColumnsRowsOrDiagonals = (array) => {
    const states = array.reduce((previousValue, row, index) => {
        // If there is at least one draw on the row/column/diagonal
        // the row/column/diagonal is a draw
        const haveDraw = row.some((tile) => tile.state === types_1.TileState.Draw);
        if (haveDraw) {
            previousValue[index] = true;
            return previousValue;
        }
        const moves = row.filter((tile) => tile.state === types_1.TileState.Player1 || tile.state === types_1.TileState.Player2);
        // If there are less than 2 moves in this line,
        // this line can't be a draw
        if (moves.length < 2) {
            return previousValue;
        }
        // If every moves are equal (played by the same player)
        // this line can't be a draw
        if (moves.every((move) => move.state === moves[0].state)) {
            return previousValue;
        }
        previousValue[index] = true;
        return previousValue;
    }, new Array(array.length).fill(false));
    if (states.every((state) => state)) {
        return true;
    }
    return false;
};
const checkIfDraw = (section) => {
    let array2D;
    if (!(0, checkIfSection_1.default)(section)) {
        throw new Error('section should be valid.');
    }
    if (section.length === 3) {
        array2D = [...section];
    }
    else {
        array2D = (0, convertTo2DArray_1.default)(section);
    }
    // Check if there is a draw only
    // if there is at least 8 moves in the section
    // or if there is at least 3 draw in the section
    const numOfDraw = (0, convertTo1DArray_1.default)((0, mapMatrix_1.default)(array2D, (tile) => tile.state)).filter((tile) => tile === types_1.TileState.Draw).length;
    const numOfMoves = (0, convertTo1DArray_1.default)((0, mapMatrix_1.default)(array2D, (tile) => tile.state)).filter((tile) => tile === types_1.TileState.Player1 || tile === types_1.TileState.Player2).length;
    if (numOfDraw < 3 && numOfMoves < 6) {
        return false;
    }
    // Check if all rows and all columns are equal
    const diagonalTLBR = [array2D[0][0], array2D[1][1], array2D[2][2]];
    const diagonalTRBL = [array2D[0][2], array2D[1][1], array2D[2][0]];
    const diagonals = [diagonalTLBR, diagonalTRBL];
    const rowsAreAllDraw = checkColumnsRowsOrDiagonals(array2D);
    const columnsAreAllDraw = checkColumnsRowsOrDiagonals((0, transposeMatrix_1.default)(array2D));
    const diagonalsAreAllDraw = checkColumnsRowsOrDiagonals(diagonals);
    if (rowsAreAllDraw && columnsAreAllDraw && diagonalsAreAllDraw) {
        return true;
    }
    return false;
};
exports.default = checkIfDraw;
//# sourceMappingURL=checkIfDraw.js.map