"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const convertTo2DArray_1 = __importDefault(require("./convertTo2DArray"));
const transposeMatrix_1 = __importDefault(require("./transposeMatrix"));
const checkIfSection_1 = __importDefault(require("./checkIfSection"));
const convertTo1DArray_1 = __importDefault(require("./convertTo1DArray"));
const mapMatrix_1 = __importDefault(require("./mapMatrix"));
var RowOrColumn;
(function (RowOrColumn) {
    RowOrColumn[RowOrColumn["Row"] = 0] = "Row";
    RowOrColumn[RowOrColumn["Col"] = 1] = "Col";
})(RowOrColumn || (RowOrColumn = {}));
const checkColumnsOrRows = (array, rowOrColumn) => {
    const state = array.reduce((previousValue, row, index) => {
        // If loop has already find a wining row/col,
        // return.
        if (previousValue[0] !== types_1.TileState.Empty) {
            return previousValue;
        }
        // Continue if A === B && B === C
        // and A !== Empty || A !== Draw.
        if (row[0].state === row[1].state &&
            row[1].state === row[2].state &&
            row[0].state !== types_1.TileState.Empty &&
            row[0].state !== types_1.TileState.Draw) {
            // Check if we check the columns or the row,
            // and assign the winning player and the position
            // of the row/col.
            if (index === 0) {
                const place = rowOrColumn === RowOrColumn.Row
                    ? types_1.WinningLine.TopRow
                    : types_1.WinningLine.LeftColumn;
                return [row[0].state, place];
            }
            else if (index === 1) {
                const place = rowOrColumn === RowOrColumn.Row
                    ? types_1.WinningLine.MiddleRow
                    : types_1.WinningLine.MiddleColumn;
                return [row[0].state, place];
            }
            else {
                const place = rowOrColumn === RowOrColumn.Row
                    ? types_1.WinningLine.BottomRow
                    : types_1.WinningLine.RightColumn;
                return [row[0].state, place];
            }
        }
        // If no wining row/col has been found
        // return empty.
        return [types_1.TileState.Empty, null];
    }, [types_1.TileState.Empty, null]);
    return state;
};
const checkIfWon = (section) => {
    let array2D;
    let numOfPlayer1Tiles;
    let numOfPlayer2Tiles;
    if (!(0, checkIfSection_1.default)(section)) {
        throw new Error('section should be valid.');
    }
    if (section.length === 3) {
        array2D = [...section];
        numOfPlayer1Tiles = (0, convertTo1DArray_1.default)((0, mapMatrix_1.default)(section, (tile) => tile.state)).filter((tile) => tile === types_1.TileState.Player1).length;
        numOfPlayer2Tiles = (0, convertTo1DArray_1.default)((0, mapMatrix_1.default)(section, (tile) => tile.state)).filter((tile) => tile === types_1.TileState.Player2).length;
    }
    else {
        array2D = (0, convertTo2DArray_1.default)(section);
        numOfPlayer1Tiles = section.filter((tile) => tile === types_1.TileState.Player1).length;
        numOfPlayer2Tiles = section.filter((tile) => tile === types_1.TileState.Player2).length;
    }
    if (numOfPlayer1Tiles < 3 && numOfPlayer2Tiles < 3) {
        return [types_1.TileState.Empty, null];
    }
    // Check if a row is wining.
    const stateRows = checkColumnsOrRows(array2D, RowOrColumn.Row);
    if (stateRows[0] !== types_1.TileState.Empty) {
        return stateRows;
    }
    // Check if a column is wining.
    const transposeArray2D = (0, transposeMatrix_1.default)(array2D);
    const stateColumns = checkColumnsOrRows(transposeArray2D, RowOrColumn.Col);
    if (stateColumns[0] !== types_1.TileState.Empty) {
        return stateColumns;
    }
    // Check if a diagonal is wining.
    if (array2D[0][0].state === array2D[1][1].state &&
        array2D[1][1].state === array2D[2][2].state &&
        array2D[0][0].state !== types_1.TileState.Empty &&
        array2D[0][0].state !== types_1.TileState.Draw) {
        return [array2D[0][0].state, types_1.WinningLine.TopLeftBottomRightDiagonal];
    }
    if (array2D[0][2].state === array2D[1][1].state &&
        array2D[1][1].state === array2D[2][0].state &&
        array2D[0][2].state !== types_1.TileState.Empty &&
        array2D[0][2].state !== types_1.TileState.Draw) {
        return [array2D[0][2].state, types_1.WinningLine.TopRightBottomLeftDiagonal];
    }
    return [types_1.TileState.Empty, null];
};
exports.default = checkIfWon;
//# sourceMappingURL=checkIfWon.js.map