"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const checkIfHistory_1 = __importDefault(require("./checkIfHistory"));
const checkIfMode_1 = __importDefault(require("./checkIfMode"));
const checkIfSectionIsFull_1 = __importDefault(require("./checkIfSectionIsFull"));
const checkIfWon_1 = __importDefault(require("./checkIfWon"));
const getSections_1 = __importDefault(require("./getSections"));
const getTileIndexPositionAndSection_1 = __importDefault(require("./getTileIndexPositionAndSection"));
const getActiveSection = (history, mode = types_1.Mode.Normal) => {
    // Check if given props are valids
    if (!(0, checkIfHistory_1.default)(history)) {
        throw new Error('history should be valid');
    }
    if (!(0, checkIfMode_1.default)(mode)) {
        throw new Error('mode should be valid');
    }
    // If is the first move,
    // return null (current player can play everywhere)
    if (history.length === 0) {
        return null;
    }
    // Get position of the last play ...
    const { position } = (0, getTileIndexPositionAndSection_1.default)(history[history.length - 1]);
    const sections = (0, getSections_1.default)(history);
    const index = sections.findIndex((item) => item.position === position);
    // Should never be reached ...
    if (index === -1) {
        throw new Error('position not found');
    }
    // If the section which is supposed to be active is full, return null
    if ((0, checkIfSectionIsFull_1.default)(sections[index].tiles)) {
        return null;
    }
    // If mode === normal and the active section is already won,
    // current player can play everywhere else
    if (mode === types_1.Mode.Normal) {
        const sectionState = (0, checkIfWon_1.default)(sections[index].tiles);
        if (sectionState[0] !== types_1.TileState.Empty) {
            return null;
        }
    }
    return index;
};
exports.default = getActiveSection;
//# sourceMappingURL=getActiveSection.js.map