"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const convertTo2DArray_1 = __importDefault(require("./convertTo2DArray"));
const checkIfHistory_1 = __importDefault(require("./checkIfHistory"));
const generateBoardFromHistory_1 = __importDefault(require("./generateBoardFromHistory"));
const getSections = (history) => {
    if (!(0, checkIfHistory_1.default)(history)) {
        throw new Error('board should be valid.');
    }
    const board = (0, generateBoardFromHistory_1.default)(history);
    const array2D = (0, convertTo2DArray_1.default)(board);
    // Representation of a scan
    // of each sub board that might be splitted
    const slices = [
        [0, 2, 0, 2, types_1.Position.TopLeft],
        [3, 5, 0, 2, types_1.Position.TopMiddle],
        [6, 9, 0, 2, types_1.Position.TopRight],
        [0, 2, 3, 5, types_1.Position.MiddleLeft],
        [3, 5, 3, 5, types_1.Position.MiddleMiddle],
        [6, 9, 3, 5, types_1.Position.MiddleRight],
        [0, 2, 6, 9, types_1.Position.BottomLeft],
        [3, 5, 6, 9, types_1.Position.BottomMiddle],
        [6, 9, 6, 9, types_1.Position.BottomRight],
    ];
    // Split 9x9 board to 9 different slices
    const sections = slices.reduce((rows, key) => {
        const section = array2D
            .slice(key[2], key[3] + 1)
            .map((boardIndex) => boardIndex.slice(key[0], key[1] + 1));
        rows.push({
            tiles: section,
            position: key[4],
        });
        return rows;
    }, []);
    return sections;
};
exports.default = getSections;
//# sourceMappingURL=getSections.js.map