"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const checkIfAssets_1 = __importDefault(require("./checkIfAssets"));
const checkIfDraw_1 = __importDefault(require("./checkIfDraw"));
const checkIfTileBelongToSection_1 = __importDefault(require("./checkIfTileBelongToSection"));
const checkIfWon_1 = __importDefault(require("./checkIfWon"));
const getActiveSection_1 = __importDefault(require("./getActiveSection"));
const getSections_1 = __importDefault(require("./getSections"));
const getTileIndexPositionAndSection_1 = __importDefault(require("./getTileIndexPositionAndSection"));
const arrayEquals = (firstArray, secondArray) => Array.isArray(firstArray) &&
    Array.isArray(secondArray) &&
    firstArray.length === secondArray.length &&
    firstArray.every((val, index) => val === secondArray[index]);
const play = (tile, assets) => {
    if (tile < 0 || tile >= 81) {
        throw new Error('tile out of bound');
    }
    if (!(0, checkIfAssets_1.default)(assets)) {
        throw new Error('assets should be valid');
    }
    if (assets.history.includes(tile)) {
        throw new Error('invalid move');
    }
    const activeSection = (0, getActiveSection_1.default)(assets.history, assets.mode);
    if (!(0, checkIfTileBelongToSection_1.default)(tile, activeSection)) {
        throw new Error('invalid move');
    }
    // Update history
    const history = [...assets.history, tile];
    // If current section is already won, update only history
    const { section: sectionIndex } = (0, getTileIndexPositionAndSection_1.default)(tile);
    const sectionStates = [...assets.sectionStates];
    let winner = assets.winner;
    if (sectionStates[sectionIndex][0] !== types_1.TileState.Empty) {
        return {
            history,
            mode: assets.mode,
            sectionStates,
            winner,
        };
    }
    const sections = (0, getSections_1.default)(history);
    const { tiles } = sections[sectionIndex];
    const sectionIsWin = (0, checkIfWon_1.default)(tiles);
    // Update winner && sectionStates only if the move change the current sectionState.
    if (sectionIsWin[0] !== types_1.TileState.Empty) {
        sectionStates[sectionIndex] = sectionIsWin;
        winner = (0, checkIfWon_1.default)(sectionStates.map((sectionState) => sectionState[0]));
        // If the section is not won by the move, check if the section might be a draw
    }
    else if ((0, checkIfDraw_1.default)(tiles)) {
        sectionStates[sectionIndex] = [types_1.TileState.Draw, null];
    }
    // If the game is not won by the move, and the state of the current section has changed,
    // Check if the game is a draw
    if (winner[0] === types_1.TileState.Empty &&
        !arrayEquals(assets.sectionStates[sectionIndex], sectionStates[sectionIndex])) {
        const gameIsADraw = (0, checkIfDraw_1.default)(sectionStates.map((sectionState) => sectionState[0]));
        if (gameIsADraw) {
            winner = [types_1.TileState.Draw, null];
        }
    }
    return {
        history,
        mode: assets.mode,
        sectionStates,
        winner,
    };
};
exports.default = play;
//# sourceMappingURL=play.js.map